/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv_gpio.h"            */
/*  Constants and definitions for GPIO.                                      */
/*                                                                           */
/*  Author : JYA                        Last modification : 12/13/2018       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
         2  Structures and typedefs
            2.1  High level GPIO driver definitions - Gpio_def
            2.2  Low level GPIO driver interface - Gpio_desc
            2.3  GPIO pin context - Gpiopin
            2.4  GPIO controller context - Gpioctl
         3  Public variables
         4  Prototypes of public procedures
         5  RAM data to be included in board_xyz.c


*/


#ifndef DRV_GPIO_H                     /* If this file is not already        */
#define DRV_GPIO_H                     /* included, include it.              */


/*
    1    Defines
    ____________
*/

/*
   2.1  High level GPIO interface - Registers and tags
   ---------------------------------------------------
*/

#define TAG_INPUT 15                   /* For IND_REPORT events.             */


/* Controller registers .....................................................*/

#define NRTYPE               0         /* string  TYPE      (  1 UINT )      */
#define NRNAME               1         /* string  NAME      (  2 UINTs)      */
#define NRNUM                3         /* integer NUM       (  1 UINT )      */
#define NRSIGNALS            4         /* string  SIGNALS   (128 UINTS)      */
#define NRPINS             132         /* string  PINS      (128 UINTs)      */
#define NRUNAMES           260         /* string  UNAMES    (128 UINTs)      */
#define NRPOLLTIM          388         /* integer POLLTIM   (  1 UINT )      */
#define NRPOLLCNT          389         /* integer POLLCNT   (  1 UINT )      */

/* Pin registers ............................................................*/

#define NRSIGNAL           390         /* string  SIGNAL    (  2 UINTs)      */
#define NRPIN              392         /* string  PIN       (  2 UINTs)      */
#define NRUNAME            394         /* string  UNAME     (  2 UINTs)      */

#define NRFUNCS            396         /* string  SUPPFUNC  ( 16 UINTs)      */
#define NRFUNC             412         /* string  FUNC      (  4 UINT )      */
#define NRWPULL            416         /* integer WEAKKPULL (  1 UINT )      */
#define NROTYPE            417         /* integer OTYPE     (  1 UINT )      */
#define NROUT              418         /* integer OUTPUT    (  1 UINT )      */
#define NRINP              419         /* integer INPUT     (  1 UINT )      */

#define NREVENTS           420         /* integer EVENTS    (  1 UINT )      */
#define NRKEYCODE          421         /* integer KEYCODE   (  1 UINT )      */
#define NRKEYINV           422         /* integer KEYINV    (  1 UINT )      */
#define NRCNTF             423         /* integer COUNTFALL (  1 UINT )      */
#define NRCNTR             424         /* integer COUNTRISE (  1 UINT )      */
#define NRLWNUM            425         /* integer LWNUM     (  1 UINT )      */

#define NRMON              426         /* integer MONITOR   (  1 UINT )      */
#define NRDTIM             427         /* integer DIFITIM   (  1 UINT )      */
#define NRDCNT             428         /* integer DIFICNT   (  1 UINT )      */


/* Size of table of registers for controller and pin ........................*/

#define NBREGCTL             6         /* Controller registers (0 to 389)    */
#define NBREGPIN             5         /* Pin registers (388 to 419)         */


/*
   2.2  Low level GPIO interface - Mnemonics for function parameters
   -----------------------------------------------------------------
*/

/* GPIO static functions ....................................................*/

#define FNC_GPIO             0         /* Function GPIO (0). Mandatory.      */

/* GPIO output configuration ................................................*/

#define GP_LOW               0         /* Low (-V or ground).                */
#define GP_HIGH              1         /* High (+V).                         */
#define GP_HIZ               2         /* High impedance (floating).         */

/* GPIO weak-pull configuration .............................................*/

#define GP_NP                0         /* No pull-up or pull-down.           */
#define GP_PU                1         /* Weak pull-up.                      */
#define GP_PD                2         /* Weak pull-down.                    */

/* GPIO output push-pull/open-drain configuration ...........................*/

#define GP_PP                0         /* Push-pull                          */
#define GP_OD                1         /* Open-drain                         */



/*
    2    Structures and typedefs
    ____________________________
*/


/*
   2.1  High level GPIO driver definitions - Gpio_pindef
   -----------------------------------------------------
*/


typedef struct                         /* GPIO pin init.                     */
  BEGIN                                /* ---------------------------------- */
    const CHAR* const sname          ; /* GPIO signal name                   */
    const CHAR* const uname          ; /* GPIO user name (alias)             */
    UCHAR             hidden         ; /* Hidden flag                        */
    const CHAR* const func           ; /* Function                           */
    UCHAR             pull           ; /* Pull configuration                 */
    UCHAR             otype          ; /* push-pull/open_drain configuration */
    UCHAR             outcfg         ; /* Output configuration               */
  END                 Gpio_pindef    ; /* ---------------------------------- */


/*
   2.2  Low level GPIO driver interface - Gpio_desc
   ------------------------------------------------
*/


typedef struct                         /* Low-level procedure's addresses    */
  BEGIN                                /* ---------------------------------- */
    void            (*info       )() ; /* Get name tables addresses          */
    void            (*get_cap    )() ; /* Get one pin capacities             */
    void            (*init       )() ; /* Hardware initialization            */
    void            (*set_func   )() ; /* Select one function                */
    void            (*get_input  )() ; /* Read one pin state                 */
    void            (*set_output )() ; /* Set one output pin state           */
    void            (*set_outcfg )() ; /* Set one pin configuration          */
    void            (*get_outcfg )() ; /* Get one pin configuration          */
    void            (*set_pull   )() ; /* Set the weak pull configuration    */
    void            (*get_pull   )() ; /* Get the weak pull configuration    */
    void            (*set_otype  )() ; /* Set the output in push-pull/O drain*/
    void            (*get_otype  )() ; /* Get the output push-pull/Open drain*/
    int             (*set_difi   )() ; /* Set the digital filter for one pin */
    int             (*set_irq    )() ; /* Set the IRQ for one pin            */
  END               Gpio_desc        ; /* ---------------------------------- */


/*
   2.3  GPIO pin context - Gpiopin
   -------------------------------
*/


typedef struct                         /* GPIO pin context                   */
  BEGIN                                /* ---------------------------------- */
    Gpio_desc      *desc             ; /* Low-level driver interface.        */
    USHORT          vl               ; /* Logical channel in AUT_GPIO.       */
                                       /* ---------------------------------- */
    UINT            iodsub           ; /* Subchannel IOD.                    */
    USHORT          reqemet          ; /* Allocator automaton.               */
    USHORT          reqevl           ; /* Allocator logical channel          */
    USHORT          reqnode          ; /* Allocator node.                    */
                                       /* ---------------------------------- */
    UCHAR           dev              ; /* High level controller number       */
    UCHAR           bank             ; /* GPIO bank number for the pin       */
    UCHAR           pin              ; /* Pin number for its bank            */
                                       /* ---------------------------------- */
    UINT            pindef           ; /* Default value for reg[0]           */
    UINT            reg[NBREGPIN]    ; /* Missed events counter.             */
    USHORT          pnext            ; /* Next pin context to be polled      */
    UINT            pcnt             ; /* Successive identical polls count   */
    UCHAR           psamp            ; /* Last sampled value                 */
  END               Gpiopin_ctx      ; /* ---------------------------------- */


/*
   2.4  GPIO controller context - Gpioctl
   --------------------------------------
*/


typedef struct                         /* GPIO controller context            */
  BEGIN                                /* ---------------------------------- */
    Req_ctx         rctx             ; /* Allocation context.                */
                                       /* Returned by infoGPIO ------------- */
    CHAR          **funcs            ; /* Table of supported functions.      */
    INT             nbbanks          ; /* Number of GPIO banks               */
    CHAR         ***signals          ; /* Signal names for each bank         */
    CHAR         ***pins             ; /* Pin    names for each bank         */
    Gpio_pindef    *pindef           ; /* User   names and pin configuration */
                                       /* Registers storage ---------------- */
    UINT            reg[NBREGCTL]    ; /* Missed events counter.             */
    UINT            idto             ; /* Pin state Polling timeout          */
                                       /* ---------------------------------- */
    Gpiopin_ctx    *pin_ctx          ; /* Adress of table of pin's contexts  */
    INT             nbpins           ; /* Number of pins/ context pins       */
    INT             nballoc          ; /* Count of allocated pins            */
    USHORT          pfirst           ; /* First pin context to poll          */
  END               Gpioctl_ctx      ; /* ---------------------------------- */
     

/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


IWORD search_gpio_pindef    (CHAR*,Gpio_desc*,INT*,INT*,INT*,Gpio_pindef**   );
void  search_gpio_idx       (const CHAR*,const CHAR*,INT*,INT*               );
IWORD switch_gpio           (const Gpio_pindef*,INT,INT                      );
void  sig_input             (INT,INT                                         );


#endif                                 /* End of DRV_GPIO_H.                 */

/*
    5  RAM data to be included in board_xyz.c
    _________________________________________
*/

#ifdef BOARD_XYZ_DATA_C

const USHORT gpio_nbctrl = GPIO_NBCTL; /* Number of LW for controllers       */
const USHORT gpio_nbiod = GPIO_NBIOD ; /* Controllers seen by the VMOS       */
const USHORT gpio_nbvl = GPIO_NBVL   ; /* Number of logical channels.        */
const USHORT gpio_nbpin = GPIO_NBPIN ; /* Number of pin context's            */

Gpioctl_ctx  gpioctl_ctx[GPIO_NBCTL] ; /* Controller's context table.        */
Gpiopin_ctx  gpiopin_ctx[GPIO_NBPIN] ; /* Pins' contexts table.              */
USHORT       gpio_states[GPIO_NBVL]  ; /* Logical Ways states.               */

Itf_ctx gpio_itfctx[1+GPIO_NBIOD]    ;
USHORT  gpio_sub_vl[GPIO_NBPIN]      ;


#endif                                 /* End of BOARD_XYZ_C                 */
