/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv_rcu.h"             */
/*  Constants and definitions for RCU driver.                                */
/*                                                                           */
/*  Author : JYA                        Last modification : 11/08/2019       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1  Receiver types
            1.2  Carrier modulation code
            1.3  Protocols codes
            1.4  Bits encoding types codes
            1.5  Special symbols identifiers
            1.6  Values for the "format" parameter
         2  Structures and typedefs
            2.1  Symbols description - Rcu_descsym and Rcu_sym
            2.2  RCU key description - Rcu_key
            2.3  RCU message description - Rcu_mess
            2.4  RCU description - Rcu_def
            2.5  High level driver parameters - Rcu_drv_def
            2.6  Low-level interface procedures - Rcu_desc
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef DRV_RCU_H                      /* If this file is not already        */
#define DRV_RCU_H                      /* included, include it.              */

#include <board.h>                     /* Needed by req_lib.h                */
#include <req_lib.h>                   /* Needed by drv_gpio.h               */
#include <drv_gpio.h>                  /* Needed by Rcu_drv_def              */

/*
    1    Defines
    ____________
*/

/*  1.1  Receiver types
    -------------------
*/

#define RCU_NONE              0        /* Power off the receiver             */
#define RCU_WIRE              1        /* Select baseband input              */
#define RCU_IR                2        /* Select IR input                    */
#define RCU_UHF               3        /* Select UHF input                   */
#define RCU_MAX               4        /* Count                              */


/*  1.2  Carrier modulation code
    ----------------------------
*/

#define MOD_NONE              0        /* No carrier - No modulation         */
#define MOD_SQA13OO           1        /* Square Form Factor 1/3 ON OFF      */
#define MOD_ASKOO             2        /* Sine Amplitude Shift Keying ON OFF */
#define MOD_FSK2              3        /* Sine Frequency Shift Keying +-f    */


/*  1.3  Protocols codes
    --------------------
*/

#define PRO_RAW               0        /* Symbols durations raw              */
#define PRO_SYMB              1        /* Symbols numbers                    */
#define PRO_NEC               2        /* Tail codes                         */
#define PRO_RPT               3        /* Repeat messages                    */
#define PRO_RC6               4        /* RC6 mode 0                         */
#define PRO_RC5               5        /* RC5                                */
#define PRO_XMP               6        /* Universal Electronics XMP          */
#define PRO_CMM               7        /* RCMM                               */
#define PRO_HE                8        /* HomeEasy protocol (32 bits)        */
#define PRO_HEL               9        /* HomeEasy protocol (36 bits)        */
#define PRO_DCF              10        /* DCF77                              */
#define PRO_NB               11        /* Protocols count                    */


/*  1.4  Bit encoding type codes
    ----------------------------
*/

#define COD_UNKNOWN           0        /* To be used with PRO_RAW            */
#define COD_OFFDUR            1        /* OFF duration encoding              */
#define COD_OFFMAN            2        /* OFF1-OFF2 durations encoding       */
#define COD_RC5               3        /* RC5 encoding                       */
#define COD_RC6               4        /* RC6 encoding                       */
#define COD_HE                5        /* HomeEasy protocol (32 bits)        */
#define COD_HEL               6        /* HomeEasy protocol (36 bits)        */


/*  1.5  Special symbols identifiers
    --------------------------------
*/
#define SYM_LEADER           -1        /* Leader      symbol identifier      */
#define SYM_DATFIRST         -2        /* First data  symbol identifier      */
#define SYM_DATLAST          -3        /* Last  data  symbol identifier      */
#define SYM_TAIL             -4        /* Tail code   symbol identifier      */
#define SYM_PSTOP            -5        /* Last  pulse symbol identifier      */
#define SYM_FPULSE           -6        /* First pulse symbol identifier      */
#define SYM_LASTID           -7        /* LAST IDENTIFIER value              */


/*  1.6  Values for the "format" parameter
    --------------------------------------
*/
#define FMT_RAW               0        /* Pairs of (ON,OFF) USHORT's         */
#define FMT_SYMB              1        /* One byte symbol codes              */
#define FMT_BIT               2        /* One bit per byte                   */
#define FMT_BYTE              3        /* Eight bits per byte                */
#define FMT_KEYCODE           4        /* "res2" is the keycode value        */


/*
    2    Structures and typedefs
    ____________________________
*/

/*  2.1  Symbols description - Rcu_descsym and Rcu_sym
         ----------------------------------------------
*/

typedef struct                         /* Timings for ONE symbol             */
  BEGIN                                /* ---------------------------------- */
    USHORT          otmin            ; /* on     time minimal in scale unit  */
    USHORT          otmax            ; /* on     time maximal in scale unit  */
    USHORT          stmin            ; /* symbol time minimal in scale unit  */
    USHORT          stmax            ; /* symbol time maximal in scale unit  */
    USHORT          otnom            ; /* on     time nominal in scale unit  */
    USHORT          stnom            ; /* symbol time nominal in scale unit  */
  END               Rcu_sym          ; /* ---------------------------------- */

typedef struct                         /* Parameters for one symbol list     */
  BEGIN                                /* ---------------------------------- */
    CHAR            scale            ; /* Time scale factor -6->usec -3->msec*/
    UCHAR           flags            ; /* List of option bits                */
                                       /*                                    */
    UCHAR           nbsym            ; /* Count of symbols in the list       */
    UCHAR           leader           ; /* Leader symbol index                */
    UCHAR           datfirst         ; /* First data symbol index            */
    UCHAR           datlast          ; /* Last  data symbol index            */
    UCHAR           tail             ; /* Tail symbol index                  */
    UCHAR           pstop            ; /* STOP pulse symbol index            */
    UCHAR           bitpsym          ; /* Count of bits per each data symbol */
    UCHAR           fpulse           ; /* Symbol index to define the pulse   */
  END               Rcu_descsym      ; /* ---------------------------------- */


/*  2.2  RCU key description - Rcu_key
         -----------------------------
*/

typedef struct                         /* 1 touche d'1 telecommande          */
  BEGIN                                /* ---------------------------------- */
    USHORT          codercu          ; /* Code recu de la telecommande       */
    USHORT          codevmos         ; /* Code transmis au VMOS              */
  END               Rcu_key          ; /* ---------------------------------- */



/*  2.3  RCU message description - Rcu_mess
    ---------------------------------------
*/

typedef struct                         /* One RCU description                */
  BEGIN                                /* ---------------------------------- */
    UCHAR           nbsymdata        ; /* Count of data symbols              */
    UCHAR           codeshft         ; /* Keycode right shift                */
    UINT            codemsk  [2]     ; /* Keycode mask                       */
    UINT            idval    [2]     ; /* RCU Identifier                     */
    UINT            idmsk    [2]     ; /* RCU Identifier mask                */
    UINT            relval   [2]     ; /* Release key indicator              */
    UINT            relmsk   [2]     ; /* Release key indicator mask         */
    const Rcu_key  *keylist          ; /* Definition keys table address      */
    UINT            torep            ; /* Message repetition duration in msec*/
    USHORT          cntrep           ; /* Repeat count (transmitter)         */
  END               Rcu_mess         ; /* ---------------------------------- */


/*  2.4  RCU description - Rcu_def
    ------------------------------
*/

typedef struct                         /* One RCU description                */
  BEGIN                                /* ---------------------------------- */
    CHAR              *name          ; /* RCU name                           */

/* Link level RCU description ...............................................*/

    UCHAR              link          ; /* RCU_BB/RCU_IR/RCU_UHF              */
    UCHAR              modul         ; /* Carrier modulation                 */
    UINT               freq          ; /* Carrier frequency in Hz            */

/* Protocol level description ...............................................*/

    UCHAR              typro         ; /* RCU protocol                       */
    UCHAR              bitenc        ; /* Bits encoding method               */
    const Rcu_descsym *descsym       ; /* Global symbols description         */
    const Rcu_sym     *sym           ; /* Symbols description list address   */
    UINT               toerr         ; /* TO attente apres erreur            */

/* Message level description ................................................*/

    const Rcu_mess    *rcu_mess      ; /* RCU keys                           */
    const Rcu_mess    *kbd_mess      ; /* KBD keys                           */
    const Rcu_mess    *mse_mess      ; /* Trackball/Mouse function           */

  END                  Rcu_def       ; /* ---------------------------------- */


/*  2.5  High level driver parameters - Rcu_drv_def
    -----------------------------------------------
*/

typedef struct                         /* Input High-level parameters        */
  BEGIN                                /* ---------------------------------- */
    UCHAR           link             ; /* RCU_BB/RCU_IR/RCU_UHF              */
    UCHAR           modul            ; /* Carrier modulation/demodulation    */
    UINT            freq             ; /* Carrier frequency                  */
    UCHAR           signal           ; /* 0:normal  1:inverted               */
    CHAR           *defrcu           ; /* Default RCU name                   */
  END               Rcu_modem        ; /* ---------------------------------- */


typedef struct                         /* Variable length list of elt's      */
  BEGIN                                /* ---------------------------------- */
    UCHAR           typdev           ; /* 0:Receiver  1:Transmitter          */
    UCHAR           defswpos         ; /* Default active input (0:first)     */
    UCHAR           nbswpos          ; /* Number of switch positions         */
    const Rcu_modem   *modem         ; /* Input's description                */
    const Gpio_pindef *pindef        ; /* GPIO's  descriptions               */
  END               Rcu_drv_def      ; /* ---------------------------------- */


/*  2.6  Low-level interface procedures
    -----------------------------------
*/

typedef struct                         /* Device RCU                         */
  BEGIN                                /* ---------------------------------- */
    IWORD          (*init_rcurcv) () ; /* RCURCV initialization function     */
    void           (*end_rcurcv)  () ; /* RCURCV termination function        */
    void           (*mute_rcurcv) () ; /* Mute/Unmute a receiver             */
                                       /* ---------------------------------- */
    IWORD          (*init_rcusnd) () ; /* RCUSND initialization function     */
    void           (*end_rcusnd)  () ; /* RCUSND termination function        */
    void           (*start_rcusnd)() ; /* Starts sending a list of symbols   */
    void           (*abort_rcusnd)() ; /* Aborts an ongoing sending          */
  END               Rcu_desc         ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/

extern const Sdrv_desc sdrv_rcu      ;

/*
    4    Prototypes of public procedures
    ____________________________________
*/

IWORD  rcu_get_nb          (INT*,INT*,INT*,INT*                              );
IWORD  rcu_get_id          (INT,INT,INT*,INT*                                );
IWORD  rcu_get             (INT,INT*,INT*,CHAR*,INT*                         );
IWORD  rcu_get_link_info   (INT,CHAR*,INT*,INT*,INT*,INT*,INT*               );
IWORD  rcu_get_proto_info  (INT,CHAR*,INT*,INT*,INT*,INT*,INT*,INT*          );
IWORD  rcu_get_sym_info    (INT,INT,INT*,INT*,INT*,INT*,INT*,INT*,INT*       );
IWORD  rcu_get_swi_info    (INT,INT*,INT*,INT*                               );
IWORD  rcu_get_mod_info    (INT,INT,INT*,INT*,INT*,INT*,CHAR*,INT*           );

IWORD  rcu_set             (INT,INT,const CHAR*,UINT,INT*                    );
IWORD  rcu_mute            (INT,INT,INT,INT*                                 );
IWORD  rcu_send            (INT,UCHAR*,INT,INT,INT,INT,INT,INT*              );
IWORD  rcu_receive         (INT,Rcu_sym*,UCHAR*,INT,INT,INT,INT,INT*         );
IWORD  rcu_abort           (INT,INT,INT*                                     );
IWORD  rcu_send_keycode    (INT,INT,INT,INT*                                 );
IWORD  rcu_ecrfa           (IWORD,IWORD,IWORD,UCHAR*,IWORD,UINT              );

#endif                                 /* End of DRV_RCU_H.                  */
